import pandas as _pd
import os as _os


def path(factor):
    """
    factor - str - factor name (use names() to find out all factor names included)
    returns the file name of the factor (e.g path('Total Assets') gives 'balance')
    """
    # can be a loop or sommething
    # Still in abs path, need changes
    p = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
    path = _os.path.join(p, 'database')
    if factor in ['price_daily', 'price_monthly', 'price_weekly']:
        return factor
    balance = set(_pd.read_csv(path + '/AAPL/AAPL_balance.csv')['Statements'].tolist())
    if factor in balance:
        return 'balance'
    del balance
    income = set(_pd.read_csv(path + '/AAPL/AAPL_income.csv')['Statements'].tolist())
    if factor in income:
        return 'income'
    del income
    cash_flow = set(_pd.read_csv(path + '/AAPL/AAPL_cash_flow.csv')['Statements'].tolist())
    if factor in cash_flow:
        return 'cash_flow'
    del cash_flow
    trading = set(_pd.read_csv(path + '/AAPL/AAPL_Trading_Information.csv')['0'].tolist())
    if factor in trading:
        return 'Trading_Information'
    del trading
    financial = set(_pd.read_csv(path + '/AAPL/AAPL_Financial_Highlights.csv')['0'].tolist())
    if factor in financial:
        return 'Financial_Hightlights'
    del financial
    valuation = set(_pd.read_csv(path + '/AAPL/AAPL_Valuation_Measures.csv')['0'].tolist())
    if factor in valuation:
        return 'Valuation_Measures'
    del valuation
    summary = set(list(_pd.read_csv(path + '/AAPL/AAPL_Summary.csv')))
    if factor in summary:
        return 'Summary'
    del summary
    raise ValueError("Factor '" + factor + "' not in database")



import unittest as _unittest

class test_calculations(_unittest.TestCase):
    def test_rank(self):
        # this take a hell long time
        from .rank import factor_percentile, percentile
        self.assertGreater(len(factor_percentile('Total Revenue', 'AAPL')), 3)
        self.assertIsInstance(percentile('Total Assets', 80), list)
        with self.assertRaises(TypeError):
            factor_percentile('Total Revenue', ['AAPL', 'AMZN'])
            factor_percentile(['Total Revenue', 'Gross Profit'], 'AAPL')
            percentile([1,2,3], 10)
            percentile('Total Revenue', '80')
        with self.assertRaises(ValueError):
            percentile('Total Revenue', 120)
            percentile('lajlsdv', 99)
            factor_percentile('oidcdlewuh', 'AAPL')
            factor_percentile('Total Revenue', 'ofisuq')



import os as _os
import gc as _gc
import pandas as _pd
import numpy as _np

global p
p = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
global datapath
datapath = _os.path.join(p, 'database')


def database_count():
    """
    prints out the total number of companies and sheets in database
    """
    a = len(_os.listdir(datapath)) - 2
    b = _os.walk(datapath)  # generator
    c = [1]
    c = len([c[0] + 1 for root, dirs, files in b for name in files]) - 12
    print("Total number of companies contained: {}".format(a))
    print("Total number of detailed sheets: {}".format(c))
    _gc.collect()


def database_clear():
    """
    clear all data in database (use cautiously)
    """
    files = (i for i in _os.listdir(datapath))
    for f in files: # file name
        if f not in ['__init__.py', 'AAPL', 'AMZN', 'general', 'test']:
            _os.remove(f)
    f2 = _os.listdir(datapath)
    assert len(f2)<6
    print("Sucessfully clear all data in database")


def database_reset():
    """
    reset database to the original state
    automatically save the past database to database_cache
    """
    pass
    


def factors_names(sheet=None):
    if sheet is None:
        fil = _os.listdir(_os.path.join(datapath, 'AAPL'))
        files = fil[:]
        try:
            files.remove('__init__.py')
            files.remove('__pycache__')
        except ValueError:
            pass

        dfs = (_pd.read_csv(_os.path.join(datapath, 'AAPL', c)) for c in files)  # generator
        r = [df.iloc[0:, 0].values[1:] for df in dfs if len(df) < 48]
        r = [i.tolist()[j] for i in r for j in range(len(i.tolist()))] + list(
            _pd.read_csv(_os.path.join(datapath, 'AAPL', 'AAPL_Summary.csv')))[1:]
        r = _np.array(r, dtype='str')
    else:
        if sheet not in sheets_names():
            raise ValueError('Parameter sheet is wrong, use sheets_names() to find all sheets names')
        file = 'AAPL_{}.csv'.format(sheet)
        if sheet == 'Summary':
            r = _np.array(list(_pd.read_csv(_os.path.join(datapath, 'AAPL', 'AAPL_Summary.csv')))[1:])
        else:
            r = _pd.read_csv(_os.path.join(datapath, 'AAPL', file)).iloc[0:-1, 0].values[1:]
    _gc.collect()
    return r


def sheets_names():
    """
    returns a list of ALL sheets names (not all companies have all of them)
    """
    names = ['major_holders', 'top_institutional_holders', 'top_mutual_fund_holders',
             'Trading_Information', 'Financial_Highlights', 'Valuation_Measures',
             'Executives', 'Description',
             'Earnings_Estimate', 'Revenue_Estimate', 'Earnings_History',
             'EPS_Trend', 'EPS_Revisions', 'Growth_Estimates',
             'stats', 'statements', 'reports',
             'Executives', 'Description', 'analysis', 'Summary',
             'balance', 'cash_flow', 'income']
    _gc.collect()
    return names


def code_count(what='lines', detail=False):
    """
    prints out some of the information of the this package
    what - str - count what, default output everything - supporting -> lines, defs, chars
    detail - True / False - whether return count for each file or not
    """
    path = p
    dirs = _os.listdir(path)
    dirs = [i for i in dirs if 'database' not in i]  # get rid of database stuff
    # dirs.remove('__pycache__')
    count = {}
    count_def = {}
    count_char = {}

    for d in dirs:
        cnt = 0
        cnt_def = 0
        cnt_char = 0
        if '.py' in d and ('pyc' not in d):
            f = open(_os.path.join(path, d), mode='r')
            while True:  # reduce memory usage
                try:
                    line = f.readline()
                except UnicodeDecodeError:
                    cnt += 1
                    continue
                if not line:
                    break
                else:
                    cnt += 1
                    cnt_def += line.count('def')
                    cnt_char += len(line)

            f.close()
        elif _os.path.isdir(_os.path.join(path, d)):
            for dd in _os.listdir(_os.path.join(path, d)):
                if '.py' in dd:
                    f = open(_os.path.join(path, d, dd), mode='r')
                    while True:  # reduce memory usage
                        try:
                            line = f.readline()
                        except UnicodeDecodeError:
                            cnt += 1
                            continue
                        if not line:
                            break
                        else:
                            cnt += 1
                            cnt_def += line.count('def')
                            cnt_char += len(line)
                    f.close()
        if cnt != 0:
            count[d] = cnt
        if cnt_def != 0:
            count_def[d] = cnt_def
        if cnt_char != 0:
            count_char[d] = cnt_char
    _gc.collect()

    if what == 'lines':
        if detail:
            return count
        else:
            return sum(count.values())
    if what == 'defs':
        if detail:
            return count_def
        else:
            return sum(count_def.values())
    if what == 'chars':
        if detail:
            return count_char
        else:
            return sum(count_char.values())



Holder,Shares,Date Reported,% Out
Vanguard Total Stock Market Index Fund,"110,521,153","Sep 29, 2018",2.40%
Vanguard 500 Index Fund,"85,435,882","Sep 29, 2018",1.86%
SPDR S&P 500 ETF Trust,"50,522,136","Mar 30, 2019",1.10%
Vanguard Institutional Index Fund-Institutional Index Fund,"43,783,603","Sep 29, 2018",0.95%
"Invesco ETF Tr-Invesco QQQ Tr, Series 1 ETF","37,515,932","Mar 30, 2019",0.82%
Fidelity 500 Index Fund,"35,045,571","Mar 30, 2019",0.76%
iShares Core S&P 500 ETF,"32,044,086","Mar 30, 2019",0.70%
Vanguard Growth Index Fund,"30,726,434","Sep 29, 2018",0.67%
Vanguard Information Technology Index Fund,"19,643,735","Aug 30, 2018",0.43%
Select Sector SPDR Fund-Technology,"17,431,948","Mar 30, 2019",0.38%



import numpy as _np
import re


def _decimal(array):  # eliminate ',' in numbers
    """
    if len(array)==1:

    for i in range(len(array)):
        temp = array[i]
        if isinstance(temp, _np.ndarray) and '-' in temp:
            array[i]=_np.array(['0']*len(temp)) # must be array[i]
    """
    d = _np.vectorize(lambda x: float(re.compile(',').sub('', x)))
    return d(array)


def _money_digits(array):  # not done, probably not useful
    f = _np.vectorize(_is_wrong_digit)
    return f(array)


def _is_wrong_digit(digit):
    t = {'B': 1000000000, 'M': 1000000, 'K': 1000, '%': 0.01}
    if isinstance(digit, str):
        last = digit[-1]
        if last == '-':
            return '0.0'
        if last in ['B', 'M', 'K', '%']:
            new = str(eval(digit[0:-1]) * t[last])
            return new
        else:
            return digit


def year_convert(year, df):
    temp = lambda x, y: [i for j in x for i in y if str(j) in i]
    r = temp(year, list(df))
    return r


def factor(df, factor, year=True):
    # dimensions of f are different for 'factor' and ['factor']
    if len(df) > 5 and ('2018' not in ''.join(list(df))) and year != True:
        raise ValueError("Parameter year can only be used when factor is in non-financial sheets")
    # factor-str is probably just non-sense, because get_factors() already converts str->list
    if isinstance(factor, str):
        if year == True:  # all years
            f = _decimal(_money_digits(df[df.iloc[0:, 0].isin([factor])].values[0][1:]))
        elif year == 'NEWEST':
            f = _decimal(_money_digits(_np.array([d[1] for d in df[df.iloc[0:, 0].isin([factor])].values])))
        else:  # single factor with specified years
            years = year_convert(year, df)
            # problematic here
            f = _decimal(_money_digits(df.loc[df.Statements == factor, years].values))
    elif isinstance(factor, list):
        if year == True:
            f = _decimal(_money_digits(_np.array([d[1:] for d in df[df.iloc[0:, 0].isin(factor)].values])))
        elif year == 'NEWEST':  # multiple factors with the newest year
            f = _decimal(_money_digits(_np.array([[d[1] for d in df[df.iloc[0:, 0].isin(factor)].values]])))
        else:
            years = year_convert(year, df)
            # problematic here
            f = _decimal(_money_digits([df.loc[df.Statements == fac, years].values[0] for fac in factor]))
        # big bug here, not handling specific year list, eg. [2018, 2017]
    return f



from ..operations.Open import open_file as _open_file
from ..operations.Format import _decimal


def Beta(stock):  # Valuation Measures
    """
    Beta coefficient - a measure of the volatility of a company compared to entire market, popularly used in capital asset pricing model (CAPM)
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'Valuation_Measures')

    bt = _decimal(df[df['0'].isin(['Beta (3Y Monthly)'])].values[0][1:])
    return bt



# import os
import gc as _gc
import unittest as _unittest

from .stock.test_stock import test_stocks
from .factors.test_factors import test_factors
from .basics.test_basics import test_basics
from .calculations.test_calculations import test_calculations
from .operations.test_operations import test_operations


def test():
    # from .stock.test_stock import test_stocks
    """
    Client function for testing EVERY method in this package
    """
    _unittest.main(__name__)
    _gc.collect()



Revenue Estimate,Current Qtr. (Jun 2019),Next Qtr. (Sep 2019),Current Year (2019)
No. of Analysts,29,29,33
Avg. Estimate,53.52B,61.47B,257.31B
Low Estimate,52.52B,55.26B,251.1B
High Estimate,54.27B,64.77B,261.25B
Year Ago Sales,53.27B,62.9B,265.6B
Sales Growth (year/est),0.50%,-2.30%,-3.10%



Description
"Amazon.com, Inc. engages in the retail sale of consumer products and subscriptions in North America and internationally. The company operates through three segments: North America, International, and Amazon Web Services (AWS) segments. It sells merchandise and content purchased for resale from third-party sellers through physical stores and online stores. The company also manufactures and sells electronic devices, including Kindle e-readers, Fire tablets, Fire TVs, and Echo devices; provides Kindle Direct Publishing, an online service that allows independent authors and publishers to make their books available in the Kindle Store; and develops and produces media content. In addition, it offers programs that enable sellers to sell their products on its Websites, as well as their own branded Websites; and programs that allow authors, musicians, filmmakers, skill and app developers, and others to publish and sell content. Further, the company provides compute, storage, database, and other AWS services, as well as compute, storage, database offerings, fulfillment, publishing, digital content subscriptions, advertising, and co-branded credit card agreement services. Additionally, it offers Amazon Prime, a membership program, which provides free shipping of various items; access to streaming of movies and TV episodes; and other services. It serves consumers, sellers, developers, enterprises, and content creators. Amazon.com, Inc. has a strategic partnership with Volkswagen AG. The company was founded in 1994 and is headquartered in Seattle, Washington."



import pandas as pd
from pyquery import PyQuery as pq


def get_executives(html):
    doc = pq(html)
    text = doc('#Main table').text().split('\n')
    t = [text[i:i + 5] for i in range(0, len(text), 5)]
    columns = t.pop(0)
    df = pd.DataFrame(t)
    df.columns = columns
    return df


def get_description(html):
    doc = pq(html)
    text = doc('.quote-sub-section p').text()
    df = pd.DataFrame([text])
    df.columns = ['Description']
    return df



Holder,Shares,Date Reported,% Out
Vanguard Total Stock Market Index Fund,"10,538,608","Sep 29, 2018",2.14%
Vanguard 500 Index Fund,"7,628,516","Sep 29, 2018",1.55%
SPDR S&P 500 ETF Trust,"4,653,551","Mar 30, 2019",0.95%
Fidelity Contrafund Inc,"4,539,093","Mar 30, 2019",0.92%
Growth Fund Of America Inc,"4,298,895","Mar 30, 2019",0.87%
Vanguard Institutional Index Fund-Institutional Index Fund,"3,909,417","Sep 29, 2018",0.79%
"Invesco ETF Tr-Invesco QQQ Tr, Series 1 ETF","3,908,673","Mar 30, 2019",0.79%
Price (T.Rowe) Blue Chip Growth Fund Inc.,"3,312,100","Mar 30, 2019",0.67%
Fidelity 500 Index Fund,"3,228,066","Mar 30, 2019",0.66%
iShares Core S&P 500 ETF,"2,951,598","Mar 30, 2019",0.60%



0,1
Fiscal Year Ends,"Sep 29, 2018"
Most Recent Quarter (mrq),"Mar 30, 2019"
Profit Margin,22.12%
Operating Margin (ttm),25.34%
Return on Assets (ttm),11.54%
Return on Equity (ttm),49.13%
Revenue (ttm),258.49B
Revenue Per Share (ttm),54.15
Quarterly Revenue Growth (yoy),-5.10%
Gross Profit (ttm),101.84B
EBITDA,77.34B
Net Income Avi to Common (ttm),57.17B
Diluted EPS (ttm),11.89
Quarterly Earnings Growth (yoy),-16.40%
Total Cash (mrq),80.09B
Total Cash Per Share (mrq),17.41
Total Debt (mrq),112.63B
Total Debt/Equity (mrq),106.39
Current Ratio (mrq),1.32
Book Value Per Share (mrq),22.98



0,1
Market Cap (intraday),823.46B
Enterprise Value,902.14B
Trailing P/E,15.06
Forward P/E,14.00
PEG Ratio (5 yr expected),1.37
Price/Sales (ttm),3.19
Price/Book (mrq),7.79
Enterprise Value/Revenue,3.49
Enterprise Value/EBITDA,11.66



import requests


def getter(url, timeout=10, proxies=None, retry=True, error=True):
    # main get function for all the website getter
    # it would support proxies, multiple user agent
    # it should raise errors when failure
    # Potential problems
    UA = [
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 \
        Safari/537.36']
    headers = {
        'User-Agent': UA[0]  # Select user agent
    }
    try:
        html = requests.get(url, headers=headers, proxies=proxies, timeout=timeout).text
        error = False
    except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout):
        print("Connection Error, read time out")
        error = True
    if retry:
        if error == False:
            return html
        else:
            getter(url, timeout, error)
    else:
        return False



from pyquery import PyQuery as pq
import re
import pandas as pd


def get_stats(html):
    # parser for statistics in yahoo finance
    doc = pq(html)
    tables = doc('#YDC-Col1 table').items()  # parsing rule
    # formatting to the clean format
    t = [i.text().split('\n') for i in tables]
    t = [t[0]] + [[i[j] for i in t[1:6] for j in range(len(i))]] + [[i[j] for i in t[7:9] for j in range(len(i))]]
    assert (len(t) == 3)
    t = [[i[j:j + 2] for j in range(0, len(i), 2)] for i in t]
    for i in t:  # formatting
        for j in i:
            j[0] = re.compile(' [0-9]$').sub('', j[0])
    assert (len(t) == 3)
    dfs = [pd.DataFrame(i) for i in t]
    return dfs


def get_statements(html):
    # Not done yet, needs to format and pd
    doc = pq(html)
    items = doc('#mrt-node-Col1-1-Financials table tr').items()  # Parser, will be seperated
    result = [i.text().split('\n') for i in items if len(i.text().split('\n')) > 1]
    df = pd.DataFrame(result)
    return df


def get_reports(html):  # parser for income, cash_flow, and balance sheets
    doc = pq(html)
    items = doc('#mrt-node-Col1-1-Financials table tr').items()  # Parsing rule
    result = [i.text().split('\n') for i in items if len(i.text().split('\n')) > 1]  # formatting
    columns = result.pop(0)
    columns[0] = 'Statements'  # eliminate unnecessary info
    df = pd.DataFrame(result)
    df.columns = columns
    return df



0,1
Market Cap (intraday),897.66B
Enterprise Value,925.8B
Trailing P/E,76.12
Forward P/E,47.84
PEG Ratio (5 yr expected),0.72
Price/Sales (ttm),3.72
Price/Book (mrq),18.53
Enterprise Value/Revenue,3.83
Enterprise Value/EBITDA,29.45



import os as _os
import gc as _gc

# __author__=='Hanbo'
# __version__=='0.0.1'

# global datapath
datapath = _os.path.join(_os.path.dirname(__file__), 'database')

from .stock import financials, analysis, profile, prices
# from .stock import financials
from .stock.profile import description as desc

from .basics.stocks import stock_list

from .factors.factors import get_factors

from .operations.Save import save  # done
from .operations.Tools import (database_count, database_clear, factors_names,
                               sheets_names, code_count)
from .operations.Open import open_file
# from .operations.Open import open_general as _open_general


# calculation.key.Beta is not done

from .calculation import *
from .get import update, getLastUpdate, main_get, load_stock_list, setup

from .operations.Path import path  # not tested

from .test2 import test


if len(_os.listdir(datapath)) - 2 < 100:
    print("There is not sufficient data in the database. Use main_get() to retrieve data")

_gc.collect()



import os as _os
# import pandas as pd
import numpy as _np
from scipy import stats as _Stats

import gc as _gc

from ..operations.Format import factor as _factor
from ..operations.Path import path as _path
from ..operations.Open import open_file as _open_file

# from factors import _balance, cash_flow, key, stats

global _factor_dic
_factor_dic = {'FCF': 'cash_flow', 'IC': 'balance', 'NIBCLS': 'balance', 'Invested_Book_Capital': 'balance'}


def factor_percentile(Factor, stock):
    """
    Factor - str
    stock - company you want to check
    returns the percentile of the factor of the stock - a list of floats
    """
    if not isinstance(Factor, str):
        raise TypeError("Parameter 'factors' should a string")
    if not isinstance(stock, str):
        raise TypeError("Parameter 'stock' should a string")
    p0 = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
    p = _os.path.join(p0, 'database')
    name = _path(Factor)
    dirs = _os.listdir(p)
    d = [i for i in dirs if _os.path.isdir(_os.path.join(p, i))]
    # return d
    # i+'_'+name in set(os.listdir(os.path.join(path,i))
    r = []
    for i in d:
        try:
            df = _open_file(i, name)
            f = _factor(df, Factor)
            if len(f) != 4:
                continue
            r.append(f)
        except Exception as e:
            # input('Exception: '+str(e))
            continue
    n = _np.stack(r)
    stock_value = _factor(_open_file(stock, name), Factor)
    result = [round(_Stats.percentileofscore(n[:, i], stock_value[i]), 4) for i in range(len(stock_value))]
    del n, r, d
    _gc.collect()
    return result


def percentile(Factor, percentage=80):
    """
    Factor - str
    percentile - int, percentage you want to check
    returns a list of companies with top percentile for the factor
    """
    if not isinstance(Factor, str):
        raise TypeError("Parameter 'factors' should a string")
    if not isinstance(percentage, int):
        raise TypeError("Parameter 'percentage' should a int")
    elif not 0 <= percentage <= 100:
        raise ValueError("Parameter 'percentage' should be between 0-100")
    p = _os.path.join(_os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir)), 'database')
    flag = False  # flag for using external factors
    try:
        name = _path(Factor)
    except ValueError:
        if Factor in _factor_dic.keys():
            flag = True
            name = _factor_dic[Factor]

        else:
            raise ValueError("No support for factor '" + Factor + "'")
    dirs = _os.listdir(p)
    d = [i for i in dirs if _os.path.isdir(_os.path.join(p, i))]
    r = []
    d2 = d[:]
    for i in d2:
        try:
            df = _open_file(i, name)
            if flag:
                f = eval(name + '._' + Factor + '(df)')  # call the function
            else:
                f = _factor(df, Factor)
            if len(f) != 4:
                d.remove(i)
                continue
            else:
                r.append(f)
                # input("Appended")
        except Exception as e:
            d.remove(i)
            # print(r)
            # input('Exception: '+str(e))
    n = _np.stack(r)
    assert len(n) == len(d)
    target = [_np.percentile(n[:, i], percentage) for i in range(4)]
    result = [[d[i] for i in range(len(n)) if n[i][j] >= target[j]] for j in range(len(target))]
    return result


def _CAGR(Factor, years):  # compound anual growth rate, not developed
    pass



from ..operations.Open import open_file as _open_file
from ..operations.Trans import _t_util, _translate


def Key_Executives(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of Key Executives in yahoo finance for the company
    '''
    df=_open_file(stock, name='Executives')
    return df

def description(stock,language='en'):
    '''
    stock - a company's code (eg. AAPL)
    language - default English, choose your language (e.g zh for mandarin)
    returns the description of the company in yahoo finance
    '''
    df=_open_file(stock, name='Description')
    desc=df['Description'][0]
    if language!='en':
        desc=_translate(desc,language)
    return desc

def summary(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of Summary of the company in yahoo finance
    '''
    df=_open_file(stock, name='Summary')
    return df



import os
import pandas as pd


from ..operations.Open import open_file as _open_file


def daily(stock, start=None, end=None):
    '''
    get daily historical price
    start & end -> time, e.g 2017-01-01
    returns a csv sheet
    '''
    df=_open_file(stock, name='price_daily')
    if start is not None:
        df=df[df.Date >= start]
    if end is not None:
        df=df[df.Date <= end]
    return df

def weekly(stock, start=None, end=None):
    '''
    get weekly historical price
    start & end -> time, e.g 2017-01-01
    returns a csv sheet
    '''
    df=_open_file(stock, name='price_weekly')
    if start is not None:
        df=df[df.Date >= start]
    if end is not None:
        df=df[df.Date <= end]
    return df

def monthly(stock, start=None, end=None):
    '''
    get monthly historical price
    start & end -> time, e.g 2017-01-01
    returns a csv sheet
    '''
    df=_open_file(stock, name='price_monthly')
    if start is not None:
        df=df[df.Date >= start]
    if end is not None:
        df=df[df.Date <= end]
    return df




    



from pyquery import PyQuery as pq
import pandas as pd


def get_summary(html, stock):
    doc = pq(html)
    tables = doc('#quote-summary table').items()
    t = [i.text().split('\n') for i in tables]
    t = [i[j] for i in t for j in range(len(i))]
    columns = [t[i] for i in range(0, len(t), 2)]
    data = [[t[i] for i in range(1, len(t), 2)]]
    data[0].insert(0, stock)
    columns.insert(0, 'Stock')
    del t
    df = pd.DataFrame(data)
    df.columns = columns
    return df


def get_news(html):
    doc = pq(html)
    pass



import pandas as _pd

import math as _math


from ..operations.Open import open_file as _open_file


def stats(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with all statistics in yahoo finance for the company
    '''
    a=Valuation_Measures(stock)
    b=Financial_Highlights(stock)
    c=Trading_Information(stock)
    df=_pd.concat([a,b,c],ignore_index=True)
    return df
    


def Valuation_Measures(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with Valuation Measures in statstics
    '''
    df=_open_file(stock, name='Valuation_Measures')
    return df


def Financial_Highlights(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with Financial Highlishts in statistics
    '''
    df=_open_file(stock, name='Financial_Highlights')
    return df

def Trading_Information(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with Trading Information in statistics
    '''
    df=_open_file(stock, name='Trading_Information')
    return df

def Cash_Flow(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with cash flow statements in yahoo finance
    '''
    df=_open_file(stock, name='cash_flow')
    return df

def Balance(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with balance  statements in yahoo finance
    '''
    df=_open_file(stock, name='balance')
    return df

def Income(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet with income statements in yahoo finance
    '''
    df=_open_file(stock, name='income')
    return df



import os as _os
import pandas as _pd

p = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
global datapath
datapath = _os.path.join(p, 'database')


def save_file(df, stock, name, update=False):
    path = _os.path.join(datapath, stock, stock) + '_' + name + '.csv'

    if not update:
        pass
    elif update:
        try:
            d = _pd.read_csv(path)
            d = d[list(d)[1:]]
        except FileNotFoundError:
            d = _pd.DataFrame()
        try:
            first = df.iloc[0:, 0]
            df = _pd.concat([df[list(df)[1:]], d], axis=1)
            df = df.loc[:, ~df.columns.duplicated()]  # Drop duplicated column
        except Exception as e:
            print("Exception in save_file: " + str(e))
        try:
            columns_title = list(df)
            # temp=columns_title.pop(0)
            columns_title.sort(key=lambda x: x.split('/')[-1:-3:-1], reverse=True)  # Sort index by Y & M
            # columns_title.insert(0, temp)
            df = df.reindex(columns=columns_title)  # Swap columns
            df = _pd.concat((first, df), axis=1, ignore_index=False)
        except Exception as e:
            print("Exception in save_file: " + str(e))
    df.to_csv(path, index=False)


def save_analysis(dfs, stock):
    path = _os.path.join(_os.path.join(datapath, stock), stock)
    names = ['Earnings_Estimate', 'Revenue_Estimate', 'Ernings_History',
             'EPS_Trend', 'EPS_Revisions', 'Growth_Estimates']
    for i in range(len(dfs)):
        dfs[i].to_csv(path + '_' + names[i] + '.csv', index=False)


def save_dfs(dfs, stock, names):
    path = _os.path.join(datapath, stock, stock)
    for i in range(len(dfs)):
        dfs[i].to_csv(path + '_' + names[i] + '.csv', index=False)


def save(df, name, file_type='.csv', mode='w', prt=True, test=False):
    """
    saver of financial sheets for client
    df - pandas dataframe to save
    name - desired name
    file_type - file type, need to add '.' (e.g - .csv / .xls)
    mode - recommend not change, default 'w' (create file if not exist, cover the data with each writes)
    prt - print out result or not - Saved dataframe to {}(path)
    test - for testing only
    """
    import PySimpleGUI as sg
    # path=input("Enter your path here: ")
    if '.' not in file_type:
        raise ValueError("Parameter file_type in save() should has '.' before the file format...")
    form_rows = [[sg.Text('Choose the save path')],
                 [sg.Text('Save path: ', size=(15, 1)), sg.InputText(key='save'), sg.FolderBrowse()],
                 [sg.Submit(), sg.Cancel()]]
    if test == False:
        window = sg.Window('Save to path')
        _, values = window.Layout(form_rows).Read()
        window.Close()
        path = values['save']
    elif test == True:
        window = sg.Window('Save to path')
        window.Close()
        path = _os.path.join(datapath, 'test')
    # path=values['save']
    if path == '' or None:
        print("You didn't choose a path for saving...")
        return
    try:
        p = _os.path.join(path, name + file_type)
        df.to_csv(p, mode=mode)
    except Exception as e:
        print("Exception in client saver: " + str(e))
        return
    if prt and (not test):
        print("Saved dataframe to " + p)



Revenue Estimate,Current Qtr. (Jun 2019),Next Qtr. (Sep 2019),Current Year (2019)
No. of Analysts,39,38,43
Avg. Estimate,62.44B,67.31B,275.18B
Low Estimate,60.97B,65.49B,270.01B
High Estimate,63.76B,70.01B,283.81B
Year Ago Sales,52.89B,56.58B,232.89B
Sales Growth (year/est),18.10%,19.00%,18.20%



#from getters.getter import getter
#from operations.Save import save_file
import json
from pyquery import PyQuery as pq
import requests
import pandas as pd
import time

import aiohttp
import asyncio

url='https://finance.yahoo.com'


def setproxy(p):
    proxies={
        'http' : 'http://'+p,
        'https' : 'https://'+p
        }
    return proxies

async def getter(url,proxies,timeout=10):
    headers={
        'User-Agent':'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_0) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/73.0.3683.86 Safari/537.36'}
    #s=time.time()
    try:
        async with aiohttp.ClientSession(headers=headers,connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            # Mistake here
            try:
                r = await session.get(url,proxy=proxies['http'],timeout=timeout)
                n = r.status
            except Exception:
                raise ValueError
            finally:
                await session.close()
            
            #e=time.time()
    except (aiohttp.client_exceptions.ClientProxyConnectionError, ValueError) as e:
        n=0
    if n==200:
        return True
    else:
        return False

'''
async def proxytest(ip):
    proxies=setproxy(ip)
    url='https://httpbin.org/get'
    r= await getter(url,proxies=proxies)
    return r
'''

async def test(ip):
    url='http://www.baidu.com'
    if not isinstance(ip,str):
        return
    proxies=setproxy(ip)
    s=time.time()
    r= await getter(url,proxies=proxies)
    if r:
        e=time.time()
        #result.append([ip,str(e-s)])
        #print("Proxy working "+ip)
        result.append([ip,str(round(e-s,3))])
    #else:
#        print("Proxy failed "+ip)
        

def main(ips):
    global result
    result=[]
    url='https://www.baidu.com'
    #df=pd.read_csv('/Users/hanbo/Desktop/ML/QA/JAQK/database/proxies/kuai.csv')
    #ips=df['IP']
    tasks=[asyncio.ensure_future(test(ip)) for ip in ips for _ in range(5)]
    loop=asyncio.get_event_loop()
    loop.run_until_complete(asyncio.wait(tasks))

    df=pd.DataFrame(result)
    df.columns=['IP','Speed']
    df=df.drop_duplicates('IP')
    return df


df=pd.read_csv('/Users/hanbo/Desktop/ML/QA/JAQK/database/proxies/kuai.csv')
ips=df['IP']
d=main(ips)
#main()



import pandas as _pd
# import os as _os
import numpy as _np

from collections import Iterable as _iter

from ..operations.Open import open_file as _open_file
from ..operations.Path import path as _path
from ..operations.Format import factor as _factor
from ..operations.Format import year_convert


def get_factors(companies, factors, year='NEWEST'):
    # not handling year now
    """
    companies - list of companies
    factors - list of factors
    year - default NEWEST, list of years
    companies, factors, year - maximum two of them can be
    """
    if isinstance(companies, str):
        companies = [companies]  # string - list
    elif not isinstance(companies, _iter):
        raise TypeError("Parameter 'companies' should be iterable, not "
                        + type(companies).__name__)  # iterable -> for later listcamp
    if year != True and not isinstance(year, _iter):
        raise TypeError("Parameter 'year' should be one of 'NEWEST', True, or [year, year], not "
                        + type(year).__name__)
    if isinstance(factors, list) and isinstance(year, list):
        if len(year) != 1 and len(factors) != 1 and len(companies) != 1:
            raise ValueError(
                "When you want sheet for multiple factors with multiple companies, only a specific year is supported")
    if year != True and year != 'NEWEST' and (year[0] not in [2015, 2016, 2017, 2018, 2019]):
        raise TypeError("Parameter 'year' should be [int, int, ...] with int being a year")
    if isinstance(factors, str):
        factors = [factors]  # string - list
    paths = [_path(f) for f in factors]  # things below is for drop duplicates
    d = {}
    for i in range(len(paths)):  # Avoid crowded IO
        if d.get(paths[i], 'no') == 'no':
            d[paths[i]] = [factors[i]]
        else:
            d[paths[i]] += [factors[i]]
    # for c in companies:
    #       a=_np.concatenate(tuple([_factor(_open_file(c, k), v, False) for k, v in d.items()]),
    #                         axis = 1)
    if year != 'NEWEST' and len(factors) > 1:
        if len(companies) != 1:
            raise ValueError(
                "When you want sheet for multiple factors with multiple years, only ONE specific company is supported")
        a = _np.concatenate(tuple([_np.concatenate(
            [_factor(_open_file(companies[0], k), v, year) for k, v in d.items()]
        )]), axis=1)
    else:
        a = _np.concatenate(tuple([_np.concatenate(tuple(
            [_factor(_open_file(c, k), v, year) for k, v in d.items()]
        ), axis=1) for c in companies]))

    if year != 'NEWEST':
        # year=True or [2018, 2017]
        dff = _open_file(companies[0], list(d.keys())[0])
        if year == True:
            # year=True
            years = list(dff)[1:]
            df = _pd.DataFrame(a, index=factors)
            df.columns = years
            return df
        else:
            # year=[2018, 2017]
            years = year_convert(year, list(dff))
            df = _pd.DataFrame(a, index=factors)
            df.columns = years
            return df
    else:
        # year='NEWEST'
        df = _pd.DataFrame(a, index=companies)
        df.columns = factors
        return df
    if len(factors) > 1 and len(years) > 1:
        # multiple factors and 
        df = _pd.DataFrame(a, index=factors)
        df.columns = years
        return df

    # problem here
    # df.columns=factors # when only one factor is here, the columns name should be date rather than factors



from ..operations.Format import factor as _factor
from ..operations.Open import open_file as _open_file


def _IC(df):  # Balance
    """Invested Capital - following calculations on investopedia.com"""
    # No adjustments for cash-flow and off-balance sheet yet
    return Invested_Book_Capital(df)


def _NIBCLS(df):
    accounts_payable = _factor(df, 'Accounts Payable')  # Balance
    other_current_liabilities = _factor(df, 'Other Current Liabilities')  # Balance
    return accounts_payable + other_current_liabilities


def _Invested_Book_Capital(df):  # Balance
    total_assets = _factor(df, 'Total Assets')
    return total_assets - _NIBCLS(df)


def _CAGR():
    pass


def IC(stock):
    """
    Invested Capital - following calculations on investopedia.com
    stock - company name (e.g AAPL for apple inc.)
    """
    return Invested_Book_Capital(stock)


def Invested_Book_Capital(stock):
    """
    Invested Book Capital - a component for calculating invest capital
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'balance')
    return _Invested_Book_Capital(df)


def NIBCLS(stock):
    """
    NIBCLS - account payable + other current liabilities
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'balance')
    return _NIBCLS(df)


def Total_Assets(stock):
    """
    Total Assets - resource with economic value that the company owns with the expectation of its future benefit
    stock - company name (e.g AAPL for apple inc.)
    """
    return _factor(_open_file(stock, 'balance'), 'Total Assets')


# the following will be used for the class-base module
'''
class balance():
    def __init__(self,stock):
        self.__stock=stock
        self.__df=_open_file(stock,'balance')
    def Total_Assets(self):
        return _factor(self.__df, 'Total Assets')
    def Accounts_Payable(self):
        return _factor(self.__df, 'Accounts Payable')
    def Long_Term_Debt(self):
        return _factor(self.__df, 'Long Term Debt')
    def Total_Liabilities(self):
        return _factor(self.__df, 'Total Liabilities')
    def Net_Tangible_Assets(self):
        return _factor(self.__df, 'Net Tangible Assets')
    def Total_Stockholder_Equity(self):
        return _factor(self.__df, 'Total Stockholder Equity')
'''



from pyquery import PyQuery as pq
import pandas as pd


def get_analysis(html):
    doc = pq(html)
    tables = doc('#Main table').items()
    t = [i.text().split('\n') for i in tables]
    t_df = [[tt[i:i + 4] for i in range(0, len(tt), 5)] for tt in t]
    columns = [i.pop(0) for i in t_df]
    dfs = [pd.DataFrame(i) for i in t_df]
    for i in range(len(dfs)):
        dfs[i].columns = columns[i]
    return dfs



import requests
from pyquery import PyQuery as pq
import pandas as _pd
# import re
import time
import datetime as _dtime
import os as _os

# asynchronous coroutine
import asyncio
import aiohttp

# memory cleaning
import gc as _gc

# Internal modules
from .basics.stocks import stock_list

from .getters.get_holders import get_major_holders, get_top_institutional_and_mutual_fund_holders
from .getters.get_financials import get_stats, get_statements, get_reports
from .getters.get_profile import get_executives, get_description
from .getters.get_analysis import get_analysis
from .getters.get_summary import get_summary

from .operations.Save import save_file, save_dfs, save_analysis, datapath  # helpful operations
from .operations.Folder import create_folder, exist
from .operations.Open import open_file, open_general

global main_path
main_path = _os.path.abspath(_os.path.dirname(__file__))


async def getter(url, timeout=20, error=True, proxy=None, cnt=0):
    # main get function for all the website getter
    # it would support proxies, multiple user agent
    """
    url - target url
    timeout - default 10 second (recommend >10)
    error - Recursive error handler
    proxy - connect to proxies, should be a dic containing both http/https proxies
    """
    if cnt == 1:
        return

    proxy = proxy  # Connecting to proxy pool
    UA = [
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 \
        Safari/537.36']
    headers = {
        'User-Agent': UA[0]  # Select user agent
    }
    try:
        async with aiohttp.ClientSession(headers=headers, connector=aiohttp.TCPConnector(verify_ssl=False)) as session:
            r = await session.get(url, timeout=timeout)
            html = await r.text()
        error = False
    except (requests.exceptions.ConnectionError, requests.exceptions.ReadTimeout):
        # bug in catcher
        # print("Exception in MAIN GETTET: "+str(e)) # all clean now
        error = True
    if error == False:
        return html
    else:
        await getter(url, timeout, error, cnt=cnt + 1)


async def parse(c, names, update=False, save_mode='w'):
    urls = ['https://finance.yahoo.com/quote/' + c + '/holders?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/financials?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/balance-sheet?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/cash-flow?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/key-statistics?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/profile?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '/analysis?p=' + c,
            'https://finance.yahoo.com/quote/' + c + '?p=' + c]

    try:
        # get summary needs changes - handle NAN
        # needs bug recorder
        create_folder(c)
        if not exist(c, 'Summary', update):
            try:
                html = await getter(urls[7])
                # input("Press enter to continue")
                save_file(get_summary(html, c), c, 'Summary', update)
                # print("Saved summary")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, names[3:6], update):
            try:
                html = await getter(urls[4])
                save_dfs(get_stats(html), c, names[3:6])
                # print("Saved statistics")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, names[0:3], update):
            try:
                html = await getter(urls[0])
                save_file(get_major_holders(html), c, names[0], update)
                save_dfs(get_top_institutional_and_mutual_fund_holders(html), c,
                         [names[1], names[2]])
                # print("Saved holders")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, names[6:8], update):
            try:
                html = await getter(urls[5])
                save_dfs([get_executives(html), get_description(html)], c, names[6:8])
                # print("Saved executives and description")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, names[8:14], update):
            try:
                html = await getter(urls[6])
                save_analysis(get_analysis(html), c)
                # print("Saved analysis")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, 'income', update):
            try:
                html = await getter(urls[1])
                save_file(get_reports(html), c, 'income', update)
                # print("Saved income statement")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, 'balance', update):
            try:
                html = await getter(urls[2])
                save_file(get_reports(html), c, 'balance', update)
                # print("Saved balance sheet")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        if not exist(c, 'cash_flow', update):
            try:
                html = await getter(urls[3])
                save_file(get_reports(html), c, 'cash_flow', update)
                # print("Saved cash flow statement")
                # input("Press enter to continue")
                del html
                await asyncio.sleep(0.27)
            except Exception:
                pass
        _gc.collect()
        # print("All saved for "+c)
    except Exception as e:
        bug = [[c, e]]
        # print("Exception on "+c+": ",e)


def get_all_stocks(exchange):  # Get all stocks required using the stock_list operation
    if not (exchange != 'NYSE' or exchange != 'NASDAQ'):
        raise ValueError("Exchange should be either NYSE or NASDAQ, not: '{}'".format(str(exchange)))
    s = stock_list(exchange)['Symbol'].tolist()
    return s


def main(stocks='NYSE', update=False, batch=64):
    """
    exchange -- either NYSE or NASDAQ
    """
    if stocks in ['NYSE', 'NASDAQ']:  # load all stocks
        stocks = get_all_stocks(stocks)
    assert isinstance(stocks, list)
    # stocks=['BABA'] #for testing
    # s=['BABA','AAPL','AMZN','JD','BIDU','WB','WFC','C','JPM','DPZ','BA','CVX','LUV'] # for sample testing
    NAMES = ['major_holders', 'top_institutional_holders', 'top_mutual_fund_holders',
             'Trading_Information', 'Financial_Highlights', 'Valuation_Measures',
             'Executives', 'Description',
             'Earnings_Estimate', 'Revenue_Estimate', 'Earnings_History',
             'EPS_Trend', 'EPS_Revisions', 'Growth_Estimates',
             'stats', 'statements', 'reports',
             'Executives', 'Description', 'analysis', 'Summary']  # things that'll be updated
    len_temp = int(len(stocks))
    if len_temp < batch:
        batch = len_temp
    for i in range(0, len_temp, batch):  # Yahoo Spyder main; async main loop
        # async in 3.6, different callings in 3.7
        t1 = time.time()
        tasks = [asyncio.ensure_future(parse(c, NAMES, update=update)) for c in stocks[i:i + batch]]  # async calling
        loop = asyncio.get_event_loop()
        loop.run_until_complete(asyncio.wait(tasks))
        t2 = time.time()
        print(str(i + batch) + "/" + str(len(stocks)) + " - Total Time: " + str(t2 - t1) + 's')
        # input("Cut point check")


'''
def _speedtestf():
    import shutil
    stock='NYSE'
    t=[]
    for i in range(16,256,8):
        companies=get_all_stocks(stock)[0:i]
        s=time.time()
        main(stock, batch=i)
        e=time.time()
        t.append(round((e-s)/i, 4))
        print("batch size {}: {}s".format(i, round((e-s)/i,3)))
        for c in companies:
            try:
                shutil.rmtree(os.path.join('./database',c))
            except FileNotFoundError:
                print('FileNotFoundError: '+c)
                continue
        #input('cut point check')
        gc.collect()
        time.sleep(1)
    return t
'''


def main_get(stocks='ALL', batch=32):
    """
    Main getter for client, MUST be runned after installation of the package (default update all stocks in NYSE and NASDAQ)
    stocks - str - default ALL, can be NYSE, NASDAQ, list of ticket, load_stock_list() (for client only)
    batch - default 32, batch size for loop (recommend to change based on interest status)
    """
    if stocks not in ['NYSE', 'NASDAQ', 'ALL']:
        if isinstance(stocks, str):
            t = type(stocks)
            if len(stocks) > 10:
                stocks = str(stocks[0:4] + ['......'] + stocks[-2:])
            raise ValueError("Parameter 'stocks' should be one of NYSE, NASDAQ, and ALL, not {} object: {}"
                             .format(t.__name__, str(stocks)))
    if stocks == 'ALL':
        main(stocks='NYSE', batch=batch)
        print("Updated NYSE data")
        main(stocks='NASDAQ', batch=batch)
        print("Updated NASDAQ data")
    else:
        main(stocks=stocks, batch=batch)
        if len(stocks) > 10:
            stocks = str(stocks[0:4] + ['......'] + stocks[-2:])
        print("Updated all data for" + str(stocks))


def _getBetweenDay(begin_date):  # tested
    # Got from csdn.com, minor changes have made
    begin_date = _dtime.datetime.strptime(begin_date, "%Y-%m-%d") + _dtime.timedelta(days=1)
    today = time.strftime('%Y-%m-%d', time.localtime(time.time()))
    end_date = _dtime.datetime.strptime(today, "%Y-%m-%d")
    print("Current date: " + today)
    while begin_date <= end_date:  # doesn't include today
        date_str = begin_date.strftime("%Y-%m-%d")
        yield date_str  # to reduce memory usage
        begin_date += _dtime.timedelta(days=1)


def getLastUpdate():  # get last update date of the database
    # client can access this
    """
    get last update time
    prints out the date and returns the date(str)
    """
    last_update = open(_os.path.join(main_path, 'datefile.txt')).readlines()[0]
    print("Last update time: " + last_update)
    return last_update


async def update_getter(day):  # util
    url = 'https://finance.yahoo.com/calendar/earnings?from=2019-05-12&to=2019-05-18&day={}'
    html = await getter(url.format(day), timeout=15)
    updates = [i.text()
               for i in pq(html)('.simpTblRow a').items()
               ]
    df = _pd.DataFrame(updates)
    df.to_csv('dates_temp.csv', mode='a', header=False)  # csv as a tranducer


# problem: the connection between async and normal functions
# update getter can't be connected well to the normal loop, and the current solution
# is to use a csv file as a transducer, but the method of running a singlar
# coroutine is not identified and developed, which needs to be done


async def update_stock_list(day):
    await update_getter(day)


def update_all_days():
    # Dates that need to be updated
    # Date format - YYYY-MM-DD

    # input('Cut-point check') # for checking
    # df.tolist() is depreciated... use df.values.tolist() instead
    updates = _pd.read_csv('dates_temp.csv', index_col=0).values.tolist()  # read dates
    updates = set([i[j] for i in updates for j in range(len(i))])  # for set operation AND
    needs_update_list = list(updates.intersection(stocksss))
    company_list_length = len(needs_update_list)
    print("Total update companies: {}".format(company_list_length))
    if company_list_length == 0:
        return
    try:
        main(needs_update_list, update=True)  # no syntax error here
        # working smoothly now (not fast enough)
    except Exception as e:
        print("Exception in update each day: " + str(e))


def update():
    """
    update function for JAQK package
    this automatically catches days and companies need to be updated, and update
    recommend to update every day
    """
    df = _pd.DataFrame()
    df.to_csv('dates_temp.csv')  # clear up cache
    global stocksss
    stocksss = set(_os.listdir(datapath))  # set of stocks in database
    last_update = getLastUpdate()
    days = _getBetweenDay(last_update)
    tasks = [asyncio.ensure_future(update_stock_list(day)) for day in days]  # async calling
    temp = asyncio.get_event_loop()
    temp.run_until_complete(asyncio.wait(tasks))
    print("Company list retrieved")
    update_all_days()
    #    with open('datefile.txt', mode='w') as d:
    #        d.write(time.strftime('%Y-%m-%d', time.localtime(time.time())))
    print("Update completed")


def load_stock_list():
    """
    for specific stock_list only (client's stock list: KWHS Investment Competition Approved Securities)
    """
    import PySimpleGUI as sg
    form_rows = [[sg.Text('Choose the excel path')],
                 [sg.Text('Choose path: ', size=(15, 1)), sg.InputText(key='Choose'),
                  sg.FileBrowse(file_types=(('Excel Spreadsheet', '*.xlsx'), ('Excel Spreadsheet', '*.xls')))],
                 [sg.Submit(), sg.Cancel()]]
    window = sg.Window('Choose excel from path')
    _, values = window.Layout(form_rows).Read()
    window.Close()
    path = values['Choose']  # file path chosen

    import openpyxl as xl
    wb = xl.load_workbook(path)
    sheet_names = wb.get_sheet_names()  # openpyxl for getting all sheet_names

    r = [_pd.read_excel(path, i)['TICKER'].tolist() for i in
         sheet_names]  # header of stocks is TICKER in client's stock list
    r = [i[j] for i in r for j in range(len(i))]
    return r


def _is_global(): # resolve datapath scrope problem
    try:
        type(datapath)
        return True
    except NameError:
        return False


def setup():
    """
    setup the database; this should be done before anything
    """
    assert _is_global()==True

    # setup AAPL and AMZN
    companies = ['AAPL', 'AMZN']
    dirs = [_os.listdir(_os.path.join(datapath, c)) for c in companies]
    #if not('.csv' in dirs[0][2] and '.csv' in dirs[0][5]):
    dirs2 = dirs[:]
    del dirs
    try:
        [dirs2[i].remove('__init__.py') for i in range(2)]
    except ValueError:
        pass
    if '.py' in ''.join(dirs2[0])+''.join(dirs2[1]):
        # convert .py into .csv
        [open_file(companies[c], dirs2[c][d], setup=True).to_csv(_os.path.join(datapath, companies[c], dirs2[c][d].split('.')[0]+'.csv'), index=False)
        for c in range(len(companies)) for d in range(len(dirs2[c])) if dirs2[c][d]!='__init__.py' and dirs2[c][d]!='__pycache__']
        
        # delete original .py files
        [_os.remove(_os.path.join(datapath, companies[i], dirs2[i][j])) for i in range(len(companies)) for j in range(len(dirs2[i])) if dirs2[i][j]!='__init__.py' and ('.csv' not in dirs2[i][j]) and dirs2[i][j]!='__pycache__']

    dirs_general2 = _os.listdir(_os.path.join(datapath, 'general'))
    dirs_general = dirs_general2[:]
    del dirs_general2
    try:
        dirs_general.remove('__init__.py')
    except ValueError:
        pass
    
    if '.py' in ''.join(dirs_general):
        # setup stock_list general
        exc = ['NYSE', 'NASDAQ']
        [open_general(ex, setup=True).to_csv(_os.path.join(datapath, 'general', ex+'.csv'), index=False)
         for ex in exc]
        [_os.remove(_os.path.join(datapath, 'general', ex+'.py')) for ex in exc]

    if 'dates_temp.py' in _os.listdir(main_path):
    # dates_temp.py
        _pd.read_csv(_os.path.join(main_path, 'dates_temp.py')).to_csv(_os.path.join(main_path, 'dates_temp.csv'), index=False)
        _os.remove(_os.path.join(main_path, 'dates_temp.py')) # delete original\

    if 'datefile.py' in _os.listdir(main_path):
        with open(_os.path.join(main_path, 'datefile.py')) as d:
            d = d.read()
        with open(_os.path.join(main_path, 'datefile.txt'), mode='w') as w:
            w.write(d)
        _os.remove(_os.path.join(main_path, 'datefile.py'))
        
    _gc.collect()
    print("Database has been setup")




import numpy as _np

from ..operations.Format import factor as _factor
from ..operations.Open import open_file as _open_file


def FCF(stock):  # Cash flow
    """
    FCF - free cash flow, indicating companies 'real' cash, an important valuation measure
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'cash_flow')
    OCF = _factor(df, 'Total Cash Flow From Operating Activities')
    CE = _factor(df, 'Capital Expenditures')

    return _np.subtract(OCF, CE)


'''
class cash_flow():
    def __init__(self,stock):
        self.__stock=stock
        self.__df=_open_file(stock,'cash_flow')
    def OCF(self):
        return _factor(self.__df, 'Total Cash Flow From Operating Activities')
    def CE(self):
        return _factor(self.__df, 'Capital Expenditures')
    def Net_Income(self):
        return _factor(self.__df, 'Net Income')
    def Depreciation(self):
        return _factor(self.__df, 'Depreciation')
    def Net_Borrowings(self):
        return _factor(self.__df, 'Net Borrowings')
'''



from pyquery import PyQuery as pq
import requests
import pandas as pd
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
import time

import asyncio
import aiohttp

def getter(url, timeout=10, error=True):
    # main get function for all the website getter
    # it would support proxies, multiple user agent
    # it should raise errors when failure
    # Potential problems
    proxies=None
    UA = [
        'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/72.0.3626.121 \
        Safari/537.36']
    headers = {
        'User-Agent': UA[0]  # Select user agent
    }
    try:
        html = requests.get(url,timeout=timeout).text
        error=False
    except Exception as e:
        print("Exception: ",e)
        error=True
    time.sleep(0.27)
    if error==False:
        return html
    else:
        getter(url,timeout,error)

def parse_kuai(html):
    doc=pq(html)
    a=doc('#content table').text().split('\n')[7:]
    a=[a[i:i+7] for i in range(0,len(a),7)]
    for b in a:
        b.insert(0,b.pop(0)+':'+b.pop(0))
        b[4]=float(b[4][:-1])
    df=pd.DataFrame(a)
    try:
        df.columns=['IP','Secret','Type','Location','Speed','Time']
    except ValueError:
        print("ValueError")
    return df

def parse_xici(html):
    doc=pq(html)
    a=doc('#ip_list tr').items()
    b=[i.text().split('\n') for i in a][1:]
    for i in b:
        i.insert(0,i.pop(0)+':'+i.pop(0))
    c=[[j[0]]+j[-4:] for j in b]
    assert (len(c)==len(b))
    temp={1:'b[i][1:-4]',0:'[None]'}
    c=[c[i]+eval(temp[len(b[i][1:-4])]) for i in range(len(c))]
    df=pd.DataFrame(c)
    df.columns=['IP','Secret','Type','Survive','Time','Location']
    return df
    
def parse_89(html):
    doc=pq(html)
    a=doc('table tbody tr').items()
    b=[i.text().split('\n') for i in a]
    for i in b:
        i.insert(0,i.pop(0)+':'+i.pop(0))
    df=pd.DataFrame(b)
    df.columns=['IP','Location','Network','Time']
    return df

def parse_66(urls): # This has different format than other parsers
    chrome_options=Options()
    chrome_options.add_argument('--headless')
    w=webdriver.Chrome(chrome_options=chrome_options)
    try:
        dfs=[]
        for url in urls:    
            w.get(url)
            a=w.find_element_by_css_selector('#main table').text.split('\n')[1:]
            a=[i.split(' ')[:-1] for i in a]
            for i in a:
                i.insert(0,i.pop(0)+':'+i.pop(0))
            df=pd.DataFrame(a)
            df.columns=['IP','Location','Secret','Time']
            # Could be stored seperately
            dfs.append(df)
    except Exception as e:
        print("Exception: ",e)
    finally:
        w.quit()
    return pd.concat(dfs)

def parse_ihuan(url):
    # Not usable
    # Anti spydered
    chrome_options=Options()
    chrome_options.add_argument('--headless')
    w=webdriver.Chrome(chrome_options=chrome_options)
    w.get(url)
    number=w.find_element_by_css_selector(".panel-body.form-horizontal form input[name='num']")
    number.clear()
    number.send_keys('3000')
    Type=Type=w.find_element_by_css_selector(".panel-body.form-horizontal form div select[name='anonymity'] option[value='2'")
    Type.click()
    s=w.find_element_by_css_selector(".panel-body.form-horizontal form div select[name='sort'] option[value='1']")
    s.click()
    
def parse_qy(html):
    doc=pq(html)
    a=doc('.container table tbody tr').items()
    b=[i.text().split('\n') for i in a]
    for i in b:
        i.insert(0,i.pop(0)+':'+i.pop(0))
    df=pd.DataFrame(b)
    df.columns=['IP','Secret','Type','Location','Speed','Time']
    return df

def parse_3366(html): # yun daili
    doc=pq(html)
    a=doc('#container #list table tbody tr').items()
    b=[i.text().split('\n') for i in a]
    for i in b:
        i.insert(0,i.pop(0)+':'+i.pop(0))
        i[1]='高匿代理IP'
        i.pop(3)
        i[3]=float(i[3][:-2])
    df=pd.DataFrame(b)
    df.columns=['IP','Secret','Type','Speed','Time']
    return df
    
def save(df,name):
    p='/Users/hanbo/Desktop/ML/QA/JAQK/Database/proxies/'+name+'.csv'
    try:
        df0=pd.read_csv(p)
    except FileNotFoundError:
        df0=pd.DataFrame() # possibly error
    df1=pd.concat([df,df0],ignore_index=True,sort=False).drop_duplicates(subset='IP',keep='first')
    #df2=df1.sort_values('Time',ascending=False)
    df1.to_csv(p,index=False)

def main0():
    urls=[
        'https://www.kuaidaili.com/free/inha/1/', #done
        'https://www.xicidaili.com/nn/1', #done
        'http://ip.zdaye.com/dayProxy/2019/4/1.html', # Selenium &architecture
        'http://www.89ip.cn/', #done
        'http://www.feiyiproxy.com/', # No histrocial data, needs refreshing
        'http://www.66ip.cn/1.html', #done
        'https://ip.ihuan.me/ti.html', # not usable
        'http://www.qydaili.com/free/?action=china&page=1', #done #friend link from 89ip
        'http://www.ip3366.net/free/?stype=1&page=1', # friend link of 89ip
        ]


def save_kuai():
    urls='https://www.kuaidaili.com/free/inha/'
    try:
        for i in range(1,2805):
            html= getter(urls+str(i)+'/')
            save(parse_kuai(html),'kuai')
            print("Saved "+str(i))
            #input("Press enter to continue")
    except Exception as e:
        print("Exception in main saver: ",e)
    finally:
        p='/Users/hanbo/Desktop/ML/QA/JAQK/Database/proxies/'+'kuai'+'.csv'
        df0=pd.read_csv(p)
        df1=df0.sort_values('Time',ascending=False)
        df1.to_csv(p,index=False)

def save_xici():
    url='https://www.xicidaili.com/nn/', #done
    # url+str(i)
    try:
        for i in range(1,3648):
            html=getter(url+str(i))
            save(parse_xici(html),'xici')
            print("Saved "+str(i))
    except Exception as e:
        print("Exception in main saver: ",e)
    finally:
        p='/Users/hanbo/Desktop/ML/QA/JAQK/Database/proxies/'+'xici'+'.csv'
        df0=pd.read_csv(p)
        df1=df0.sort_values('Time',ascending=False)
        df1.to_csv(p,index=False)
    
    
def main():
    '''
    for i in range(0,32,32):
        tasks=[asyncio.ensure_future(save_kuai(ii)) for ii in range(i,i+32)]
        loop=asyncio.get_event_loop()
        loop.run_until_complete(asyncio.wait(tasks))
    '''
    pass
    



from ..operations.Format import factor as _factor
from ..operations.Open import open_file as _open_file


# These are quite important and popular factors, so I list them out
# so my client can use it like ROE-ROA, instead of factor(ROE)-factor(ROA), which is quite werid
def ROE(stock):  # Financial Highlights
    """
    Return on Equity - a measure of financial performance calculated by Net Income / Shareholders' Equity
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'Financial_Highlights')
    return _factor(df, 'Return on Equity (ttm)')


def ROA(stock):  # Financial Highlights
    """
    Return on Assets - an indicator of how profitable a company is relative to its total assets
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'Financial_Highlights')
    return _factor(df, 'Return on Assets (ttm)')


def EBITDA(stock):  # Financial Highlights
    """
    Earnings before Interest Taxes Depreciation Amortization - a measure of a company's overall financial performance and is used as an alternative to simple earnings or net income
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'Financial_Highlights')
    return _factor(df, 'EBITDA')



Growth Estimates,AMZN,Industry,Sector
Current Qtr.,8.30%,N/A,N/A
Next Qtr.,15.80%,N/A,N/A
Current Year,35.40%,N/A,N/A
Next Year,39.80%,N/A,N/A
Next 5 Years (per annum),94.00%,N/A,N/A
Past 5 Years (per annum),103.71%,N/A,N/A



Growth Estimates,AAPL,Industry,Sector
Current Qtr.,-9.80%,N/A,N/A
Next Qtr.,-6.50%,N/A,N/A
Current Year,-3.50%,N/A,N/A
Next Year,11.20%,N/A,N/A
Next 5 Years (per annum),12.00%,N/A,N/A
Past 5 Years (per annum),12.32%,N/A,N/A



import os as _os
import pandas as _pd

import gc as _gc


from ..operations.Path import path as _path

p = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
global datapath
datapath = _os.path.join(p, 'database')


def open_file(stock, name, setup=False):
    """
    opener for opening sheets for client
    stock - company name (e.g AAPL for apple inc.)
    name - name of the sheet (e.g 'income' / 'balace'), use sheets_names() to see all names
    returns a csv sheet of the sheet of the company
    """
    # datapath='/Users/hanbo/Desktop/ML/QA/JAQK/database/'
    if not isinstance(stock, str):
        raise TypeError("Parameter 'stock' should be a string, not a "
                        + type(stock).__name__)
    if setup == True: # when setup, name is "AAPL_income.csv", not "income"
        path = _os.path.join(datapath, stock, name)
        df = _pd.read_csv(path)
        _gc.collect()
        return df
    names = ['major_holders', 'top_institutional_holders', 'top_mutual_fund_holders',
             'Trading_Information', 'Financial_Highlights', 'Valuation_Measures',
             'Executives', 'Description',
             'Earnings_Estimate', 'Revenue_Estimate', 'Earnings_History',
             'EPS_Trend', 'EPS_Revisions', 'Growth_Estimates',
             'stats', 'statements', 'reports',
             'Executives', 'Description', 'analysis', 'Summary',
             'balance', 'cash_flow', 'income']
    if name not in names:
        try:
            name = _path(name)
        except ValueError:
            raise ValueError(
                'Parameter "name" should be the name of the financial sheets, not a factor name...Use path method to find the location of a factor')
    path = _os.path.join(datapath, stock, stock)
    try:
        df = _pd.read_csv(path + '_' + name + '.csv')
    except FileNotFoundError:
        raise ValueError("There is no record of '" + stock + "' in database")
    _gc.collect()
    return df


def open_general(file, setup=False):
    try:
        p = _os.path.join(datapath, 'general', file)
        if setup==False:
            df = _pd.read_csv(p + '.csv')
        elif setup==True:
            df = _pd.read_csv(p + '.py')
    except Exception as e:
        print("Something wrong in opening the stock list: "+str(e))
    return df


def open_dfs(ffactor):  # Not used yet, design for reduce IOs
    """return a generator"""
    p = '/Users/hanbo/Desktop/ML/QA/JAQK/database/'
    name = _path(ffactor)
    dirs = _os.listdir(p)
    d = [i for i in dirs if
         _os.path.isdir(_os.path.join(p, i)) and _os.path.exists(_os.path.join(p, i, i) + '_' + name + '.csv')]
    print(len(d))
    dfs = (open_file(i, name) for i in d)
    return dfs



0,1
Beta (3Y Monthly),1.74
52-Week Change,12.56%
S&P500 52-Week Change,4.92%
52 Week High,"2,050.50"
52 Week Low,"1,307.00"
50-Day Moving Average,"1,881.31"
200-Day Moving Average,"1,704.60"
Avg Vol (3 month),4.35M
Avg Vol (10 day),4M
Shares Outstanding,492.33M
Float,412.73M
% Held by Insiders,16.12%
% Held by Institutions,57.12%
"Shares Short (Apr 30, 2019)",2.63M
"Short Ratio (Apr 30, 2019)",0.68
"Short % of Float (Apr 30, 2019)",0.64%
"Short % of Shares Outstanding (Apr 30, 2019)",0.53%
"Shares Short (prior month Mar 29, 2019)",4.72M



from ..operations.Format import factor as _factor
from ..operations.Open import open_file as _open_file

'''
class income:
    def __init__(self,stock):
        self.__stock=stock
        self.__df=_open_file(stock,'income')
    def Total_Revenue(self):
        return _factor(self.__df, 'Total_Revenue')
    def Cost_of_Revenue(self):
        return _factor(self.__df, 'Cost of Revenue')
    def Gross_Profit(self):
        return _factor(self.__df, 'Gross Profit')
    def Research_Development(self):
        return _factor(self.__df, 'Research Development')
    def Total_Operating_Expenses(self):
        return _factor(self.__df, 'Total Operating Expenses')
    def Interest_Expense(self):
        return _factor(self.__df, 'Interest Expense')
    def Income_Before_Tax(self):
        return _factor(self.__df, 'Income Before Tax')
    def Net_Income(self):
        return _factor(self.__df, 'Net Income')
    def Net_Income_Applicable_To_Common_Shares(self):
        return _factor(self.__df, 'Net Income Applicable To Common Shares')
'''


def Total_Revenue(stock):
    """
    Total Revenue - total money received by a company
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'income')
    return _factor(df, 'Total Revenue')


def Cost_of_Revenue(stock):
    """
    Cost of Revenue - usually used with CoR/TR (total revenue), indicating stable financial health and possible strong sales
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'income')
    return _factor(df, 'Cost of Revenue')


def Gross_Profit(stock):
    """
    Gross Profit - profits company makes after deducting the costs associated with making and selling its products 
    stock - company name (e.g AAPL for apple inc.)
    """
    df = _open_file(stock, 'income')
    return _factor(df, 'Gross Profit')



import pandas as _pd

from ..operations.Open import open_general as _open_general


def stock_list(exchange=True):
    """
    exchange - str - default True (all), either NYSE or NASDAQ
    returns a csv sheet with all statistics in yahoo finance for the company
    """
    if exchange not in ['NYSE', 'NASDAQ'] and exchange != True:
        raise ValueError("Parameter 'exchange' should either NYSE or NASDAQ")
    if exchange:
        c1 = _open_general('NASDAQ')
        c2 = _open_general('NYSE')
        df = _pd.concat([c1, c2], ignore_index=True).drop('Unnamed: 9', axis=1)  # drop duplicated column
    else:
        _csv = _open_general(exchange)
        df = _csv.drop('Unnamed: 9', axis=1)
    return df



import unittest as _unittest
# import os as _os
import numpy as _np
from .Open import open_file as _open_file
import os as _os
import pandas as _pd


class test_operations(_unittest.TestCase):
    def test_Folder(self):
        from .Folder import create_folder, p
        path = _os.path.join(p, 'operations')
        orig = _os.listdir(path)
        x = len(orig)
        if 'test_folder' not in orig:
            x += 1
        create_folder('test_folder', 'operations')
        self.assertEqual(x, len(_os.listdir(path)))  # if exists - no change

        from .Folder import exist
        self.assertTrue(exist('AAPL', 'income'))
        self.assertFalse(exist('AAPL', 'oaijc', update=True))
        self.assertTrue(exist('AAPL', ['income', 'balance', 'cash_flow']))
        self.assertFalse(exist('AAPL', ['income', 'sdojvd', 'price_monthly']))
        with self.assertRaises(TypeError):
            exist('AAPL', 123)
            exist('AAPL', True)

    def test_Format(self):
        from .Format import _decimal
        t = _decimal(['302,32', '203,2304,2034.34', '1234,2345'])
        t2 = _np.array([30232.0, 20323042034.34, 12342345.0])
        self.assertEqual(len(t), len(t2))
        [self.assertEqual(t[i], t2[i]) for i in range(len(t))]
        self.assertEqual(len(t), 3)

        from .Format import factor as _factor  # hell a lot to test
        df = _open_file('AAPL', 'income')  # AAPL

        t = _factor(df, 'Total Revenue')
        self.assertGreaterEqual(len(t), 4)
        self.assertIsInstance(t, _np.ndarray)
        (self.assertEqual(_factor(df, 'Others')[i], 0) for i in range(len(t)))

        t = _factor(df, 'Total Revenue', year='NEWEST')  # newest year
        self.assertEqual(len(t), 1)
        self.assertIsInstance(t[0], _np.float64)

        t = _factor(df, 'Total Revenue', [2018, 2017])
        self.assertEqual(len(t), 1)  # [[data]]
        self.assertEqual(len(t[0]), 2)  # two years
        self.assertIsInstance(t[0][0], _np.float64)

        t = _factor(df, ['Total Revenue', 'Gross Profit'])
        self.assertEqual(len(t), 2)
        self.assertEqual(len(t[0]), 4)
        self.assertIsInstance(t, _np.ndarray)
        self.assertIsInstance(t[0], _np.ndarray)
        self.assertIsInstance(t[0][1], _np.float64)

        t = _factor(df, ['Total Revenue', 'Gross Profit', 'Others'])  # when factor is empty
        self.assertEqual(len(t), 3)
        self.assertGreaterEqual(len(t[2]), 4)
        [self.assertEqual(t[2][i], 0) for i in range(len(t[2]))]

        t = _factor(df, ['Total Revenue', 'Gross Profit', 'Others'], 'NEWEST')
        self.assertEqual(len(t), 1)
        self.assertEqual(len(t[0]), 3)
        self.assertIsInstance(t, _np.ndarray)
        self.assertIsInstance(t[0], _np.ndarray)
        self.assertIsInstance(t[0][0], _np.float64)

        t = _factor(df, ['Total Revenue', 'Gross Profit', 'Others'], [2017, 2018])
        self.assertEqual(len(t), 3)
        self.assertEqual(len(t[0]), 2)
        self.assertIsInstance(t[0], _np.ndarray)
        self.assertIsInstance(t[0][0], _np.float64)

        from .Format import _money_digits
        a = _np.array(['123', '234K', '34%'])
        b = _np.array([['12B', '34M', '2%']])
        c = _np.array(['12345'])
        good_a = _np.array(['123', '234000', '0.34'])
        good_b = _np.array([['12000000000', '34000000', '0.02']])
        aa = _money_digits(a)
        bb = _money_digits(b)
        self.assertEqual(len(b), 1)
        [self.assertEqual(aa[i], good_a[i]) for i in range(len(a))]
        [self.assertEqual(bb[i][j], good_b[i][j]) for i in range(len(b)) for j in range(len(b[0]))]
        self.assertEqual(c, c)
        self.assertIsInstance(bb, _np.ndarray)

    def test_Open(self):
        from .Open import open_file
        df = open_file('AAPL', 'income')
        self.assertIsInstance(df, _pd.core.frame.DataFrame)
        self.assertGreaterEqual(len(list(df)), 5)
        self.assertGreaterEqual(len(df), 22)
        self.assertIn('Total Revenue', df.iloc[0:-1, 0].values)
        self.assertIn('Net Income', df.iloc[0:-1, 0].values)
        self.assertIsInstance(df.index, _pd.core.indexes.range.RangeIndex)  # if index=True

        df = open_file('AAPL', 'price_monthly')
        self.assertGreaterEqual(len(list(df)), 6)
        self.assertIsInstance(df.Open[10], _np.float64)
        self.assertIsInstance(df.Date[10], str)

        df = open_file('AAPL', 'Summary')
        self.assertGreaterEqual(len(list(df)), 16)
        self.assertEqual(df.Stock.tolist(), ['AAPL'])

        with self.assertRaises((ValueError, FileNotFoundError, TypeError)):
            open_file('AAPL', '09ioijv')
            open_file('psldnld', 'vjsdf')
            open_file(['AAPL', 'AMZN'], 'income')
            open_file(12234, 'income')

        from .Open import open_general
        self.assertGreaterEqual(len(open_general('NASDAQ')), 3000)
        self.assertGreaterEqual(len(open_general('NYSE')), 3000)

    def test_Path(self):
        from .Path import path
        self.assertEqual(path('Total Revenue'), 'income')
        self.assertEqual(path('price_daily'), 'price_daily')
        with self.assertRaises(ValueError):
            path('notindatabase')

    def test_Save(self):
        from .Save import save_file
        from .Open import open_file
        df = _pd.DataFrame([['a', 1, 2], ['b', 2, 3], ['c', 5, 6], ['d', 10, 11]])  # setup
        df.columns = ['Statements', '01/01/2018', '02/01/2017']
        from .Folder import create_folder
        create_folder('test')
        save_file(df, 'test', 'income')  # save
        df = _pd.DataFrame([['a', 12, 1], ['b', 13, 2], ['c', 14, 5], ['d', 20, 10]])  # mimic update
        df.columns = ['Statements', '03/01/2019', '01/01/2018']
        save_file(df, 'test', 'income', update=True)

        df = open_file('test', 'income')  # start testing
        self.assertEqual(len(list(df)), 4)
        self.assertEqual(list(df)[1], '03/01/2019')
        self.assertEqual(df.iloc[0:-1, 1][0], 12)

        from .Save import save, datapath
        with self.assertRaises(ValueError):
            save(df, 'name', 'csv', test=True)  # '.csv'
        save(df, 'testClientSave', test=True)
        files = _os.listdir(_os.path.join(datapath, 'test'))
        self.assertIn('testClientSave', ''.join(files))
        _os.remove(_os.path.join(datapath, 'test', 'testClientSave.csv'))

        save(df, 'testClientSave2', '.xls', test=True)
        files = _os.listdir(_os.path.join(datapath, 'test'))
        self.assertIn('.xls', ''.join(files))
        _os.remove(_os.path.join(datapath, 'test', 'testClientSave2.xls'))

    def test_Tools(self):
        # database_count() and sheets_names() no need to test
        from .Tools import factors_names
        self.assertEqual(len(factors_names()), 182)
        self.assertEqual(len(factors_names('income')), 21)
        with self.assertRaises(ValueError):
            factors_names('notindatabase')

        from .Tools import code_count
        self.assertGreaterEqual(code_count(), 1888)
        self.assertIsInstance(code_count('lines', True), dict)
        self.assertGreaterEqual(code_count('defs'), 100)
        self.assertGreaterEqual(code_count('chars'), 62888)

        # not testing database_clear() now
        



import unittest as _unittest


class test_basics(_unittest.TestCase):
    def test_stocks(self):
        from .stocks import stock_list
        self.assertGreater(len(stock_list()), 6000)
        self.assertGreater(len(stock_list('NYSE')), 3000)
        self.assertGreater(len(stock_list('NASDAQ')), 3000)
        with self.assertRaises(ValueError):
            stock_list('abcdefg')
            stock_list([1,2,3])
            stock_list(True)



import pandas as pd
from pyquery import PyQuery as pq


def get_major_holders(html):
    """Getter for holders from yahoo finance"""
    doc = pq(html)
    tables = doc('#mrt-node-Col1-1-Holders table')  # parsing rule
    t1 = pq(tables.pop(0)).text().split('\n')
    t2 = [[t1[i + 1], t1[i]] for i in range(0, len(t1), 2)]  # formatting
    df = pd.DataFrame(t2)  # convert to pd dataframe
    df.columns = ['Category', 'Percentage']
    return df


def get_top_institutional_and_mutual_fund_holders(html):
    """Getter for other holders in yahoo finance"""
    doc = pq(html)
    tables = doc('#mrt-node-Col1-1-Holders table')  # parsing rule
    # formatting
    t = [pq(i).text().split('\n') for i in tables[1:]]
    t2 = [[t1[i:i + 4] for i in range(0, len(t1), 5)] for t1 in t]
    columns = [t.pop(0) for t in t2]
    df1, df2 = pd.DataFrame(t2[0]), pd.DataFrame(t2[1])  # convert to pd dataframe
    df1.columns = columns[0]
    df2.columns = columns[1]
    return (df1, df2)



import unittest as _unittest
# import pandas as _pd
from numpy import ndarray as _ndarray


class test_factors(_unittest.TestCase):
    def test_balance(self):
        from .balance import IC, Invested_Book_Capital, NIBCLS, Total_Assets

        f = lambda c: [IC(c), Invested_Book_Capital(c),
                       NIBCLS(c), Total_Assets(c)]
        t = f('AAPL')
        self.assertEqual(len(t), 4)
        [self.assertEqual(len(_), 4) for _ in t]
        self.assertIsInstance(t[0], _ndarray)
        with self.assertRaises(ValueError):
            f('adfjsiojda')
        del t, f

    def test_cash_flow(self):
        from .cash_flow import FCF
        t = FCF('AAPL')
        self.assertEqual(len(t), 4)
        self.assertIsInstance(t, _ndarray)

        with self.assertRaises((ValueError, TypeError)):
            FCF('aosdij')  # value
            FCF(12345)  # type
            FCF(['AAPL', 'AMZN'])
            FCF(True)
        del t

    def test_income(self):
        from .income import Total_Revenue, Cost_of_Revenue, Gross_Profit

        f = lambda c: [Total_Revenue(c), Cost_of_Revenue(c), Gross_Profit(c)]
        t = f('AAPL')
        self.assertEqual(len(t), 3)
        self.assertIsInstance(t[0], _ndarray)
        self.assertGreater(len(t[1]), 3)
        with self.assertRaises(ValueError):
            f('akjdcdc')
        del t, f

    def test_stats(self):
        from .stats import ROE, ROA, EBITDA

        f = lambda c: [ROE(c), ROA(c), EBITDA(c)]
        t = f('AAPL')
        self.assertIsInstance(f('AAPL'), list)
        self.assertEqual(len(t[0]), 1)
        self.assertIsInstance(t[1], _ndarray)
        with self.assertRaisesRegex(TypeError, "Parameter 'stock' should be a string, not a list"):
            f(['AAPL', 'AMZN'])

        with self.assertRaises(ValueError):
            f('aoijdsoigjs')
        del f, t

    def test_key(self):
        from .key import Beta
        t = Beta('AAPL')
        self.assertIsInstance(t, _ndarray)

    def test_get_factors(self):
        from .factors import get_factors
        t = get_factors('AAPL', 'Gross Profit')  # 1 - 1+1
        self.assertEqual(len(t), 1)
        self.assertEqual(list(t)[0], 'Gross Profit')
        self.assertGreaterEqual(t['Gross Profit'].values[0], 100)
        self.assertEqual(t.index[0], 'AAPL')

        t = get_factors('AAPL', ['Gross Profit'])
        self.assertEqual(len(t), 1)

        t = get_factors('AAPL', ['Gross Profit', 'Total Revenue'])
        self.assertEqual(len(t), 1)
        self.assertListEqual(list(t), ['Gross Profit', 'Total Revenue'])
        self.assertEqual(t.index[0], 'AAPL')

        t = get_factors(['AAPL', 'AMZN'], 'Gross Profit')
        self.assertEqual(len(t), 2)
        self.assertListEqual(list(t), ['Gross Profit'])
        self.assertListEqual(t.index.tolist(), ['AAPL', 'AMZN'])

        t = get_factors(['AAPL', 'AMZN'], ['Gross Profit', 'Others'])
        self.assertEqual(len(t), 2)
        self.assertListEqual(list(t), ['Gross Profit', 'Others'])
        self.assertEqual(t.loc['AAPL', 'Others'], 0.0)
        self.assertGreater(t.loc['AAPL', 'Gross Profit'], 100, 'The _factor() gives wrong output dimension')

        t = get_factors('AAPL', 'Total Revenue', [2018, 2017])
        self.assertGreater(t.iloc[0, 0], t.iloc[0, 1])
        t = get_factors('AAPL', 'Others', [2018, 2016])
        self.assertListEqual(t.iloc[0, 0:].tolist(), [0, 0])
        t = get_factors('AAPL', 'Others', True)
        self.assertGreaterEqual(len(list(t)), 4)

        t = get_factors('AAPL', ['Total Revenue', 'Others', 'Total Assets'], [2018, 2017])
        self.assertListEqual(list(t), ['9/29/2018', '9/30/2017'])
        self.assertListEqual(t.index.tolist(), ['Total Revenue', 'Others', 'Total Assets'])

        t = get_factors('AAPL', ['Total Revenue', 'Others', 'Total Assets'], True)



0,1
Fiscal Year Ends,"Dec 31, 2018"
Most Recent Quarter (mrq),"Mar 31, 2019"
Profit Margin,4.97%
Operating Margin (ttm),6.17%
Return on Assets (ttm),6.12%
Return on Equity (ttm),30.06%
Revenue (ttm),241.55B
Revenue Per Share (ttm),494.21
Quarterly Revenue Growth (yoy),17.00%
Gross Profit (ttm),93.73B
EBITDA,31.44B
Net Income Avi to Common (ttm),12.01B
Diluted EPS (ttm),23.95
Quarterly Earnings Growth (yoy),118.60%
Total Cash (mrq),37.02B
Total Cash Per Share (mrq),75.19
Total Debt (mrq),69.01B
Total Debt/Equity (mrq),142.54
Current Ratio (mrq),1.09
Book Value Per Share (mrq),98.39



Description
"Apple Inc. designs, manufactures, and markets mobile communication and media devices, and personal computers. It also sells various related software, services, accessories, and third-party digital content and applications. The company offers iPhone, a line of smartphones; iPad, a line of multi-purpose tablets; and Mac, a line of desktop and portable personal computers, as well as iOS, macOS, watchOS, and tvOS operating systems. It also provides iTunes Store, an app store that allows customers to purchase and download, or stream music and TV shows; rent or purchase movies; and download free podcasts, as well as iCloud, a cloud service, which stores music, photos, contacts, calendars, mail, documents, and others. In addition, the company offers AppleCare support services; Apple Pay, a cashless payment service; Apple TV that connects to consumers' TVs and enables them to access digital content directly for streaming video, playing music and games, and viewing photos; and Apple Watch, a personal electronic device, as well as AirPods, Beats products, HomePod, iPod touch, and other Apple-branded and third-party accessories. The company serves consumers, and small and mid-sized businesses; and education, enterprise, and government customers worldwide. It sells and delivers digital content and applications through the iTunes Store, App Store, Mac App Store, TV App Store, Book Store, and Apple Music. The company also sells its products through its retail and online stores, and direct sales force; and third-party cellular network carriers, wholesalers, retailers, and resellers. Apple Inc. was founded in 1977 and is headquartered in Cupertino, California."



Earnings Estimate,Current Qtr. (Jun 2019),Next Qtr. (Sep 2019),Current Year (2019)
No. of Analysts,32,31,37
Avg. Estimate,2.11,2.72,11.49
Low Estimate,1.79,1.99,10.71
High Estimate,2.22,2.97,11.8
Year Ago EPS,2.34,2.91,11.91



import setuptools
import os
import codecs

with open("README.md", "r") as fh:
    long_description = fh.read()

def read(fname):
    return codecs.open(os.path.join(os.path.dirname(__file__), fname)).read()

def install_requirements():
    reqs = [
        'pandas>=0.21.0',
        'requests>=2.1.0',
        'numpy>=1.14.0',
        'pyquery>=1.4.0',
        'aiohttp>=3.4.0',
        'PySimpleGUI>=3.0.0',
        'scipy>=1.0.0',
        'openpyxl>=2.4.0'
        ]
    return reqs
        
        

setuptools.setup(
    name="JAQK",
    version=read('VERSION.txt'),
    author="Hanbo Guo",
    author_email="hbopublic@163.com",
    description="A light toolkit for crawling financia data and basic analysis and calcualtions.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url="https://github.com/Haannbboo/JAQK",
    install_requires=install_requirements(),
    keywords='US Stock Market Data',
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
    ],
)





# analysis
from .analysis import (Earnings_Estimate, Earnings_History, EPS_Revisions,
                       EPS_Trend, Growth_Estimates, Revenue_Estimate)

# financials
from .financials import (stats, Valuation_Measures, Financial_Highlights,
                         Trading_Information, Cash_Flow, Balance, Income)

# prices
from .prices import daily, weekly, monthly

# profile
from .profile import Key_Executives, summary



from ..operations.Open import open_file as _open_file


def Earnings_Estimate(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet pf Earnings Estimate in yahoo finance's analysis
    '''
    df=_open_file(stock, name='Earnings_Estimate')
    return df

def Earnings_History(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of Earnings History in yahoo finance's analysis
    '''
    df=_open_file(stock, name='Earnings_History')
    return df

def EPS_Revisions(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of EPS Revisions in yahoo finance's analysis
    '''
    df=_open_file(stock, name='EPS_Revisions')
    return df

def EPS_Trend(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of EPS Trend in yahoo finance's analysis
    '''
    df=_open_file(stock, name='EPS_Trend')
    return df

def Growth_Estimates(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of Growth Estimates  in yahoo finance's analysis
    '''
    df=_open_file(stock, name='Growth_Estimates')
    return df

def Revenue_Estimate(stock):
    '''
    stock - a company's code (eg. AAPL)
    returns a csv sheet of Revenue Estimate in yahoo finance's analysis
    '''
    df=_open_file(stock, name='Revenue_Estimate')
    return df



import unittest as _unittest
from .__init__ import *
from .profile import description
import pandas as _pd


class test_stocks(_unittest.TestCase):
    def test_analysis(self):
        
        f=lambda c: [EPS_Revisions(c), EPS_Trend(c),
                     Growth_Estimates(c), Revenue_Estimate(c),
                     Earnings_Estimate(c), Earnings_History(c)]
        self.assertEqual(len(f('AAPL')), 6)
        with self.assertRaises(ValueError):
            f('adfjsiojda')
       
    def test_financials(self):
        df=stats('AAPL')
        self.assertIsInstance(df, _pd.core.frame.DataFrame)
#        self.assertEqual(stats('aoidfjso'), "Something wrong with the data for stats for {}".format(stock))
        f=lambda c: {Valuation_Measures(c), Financial_Highlights(c),
                     Trading_Information(c), Cash_Flow(c),
                     Balance(c), Income(c)}
        with self.assertRaises((ValueError, TypeError)):
            f('aosdij') # value
            f(12345) # type
            f(['AAPL','AMZN'])
            f(True)
       
    def test_prices(self):
        f=lambda c, t1, t2: (daily(c), daily(c, t1), daily(c, t1, t2),
                             weekly(c), weekly(c, t1), weekly(c, t1, t2),
                             monthly(c), monthly(c, t1), monthly(c, t1, t2))
        self.assertIsInstance(f('AAPL', '2018-12-01', '2019-05-01'),  tuple)
        self.assertGreater(len(daily('AAPL', '2018-12-01', '2019-05-01')), 5) # avoid empty
        with self.assertRaises(ValueError):
            f('kjdnsdnv', '2018-12-01', '2019-05-01')
            
    def test_profile(self):
        f=lambda c: [Key_Executives(c), summary(c), description(c)]
        self.assertIsInstance(f('AAPL'), list)
#        self.assertIsInstance(description('AAPL', 'zh'), str) # translation
#        with self.assertRaisesRegex(KeyError, 'trans_result'):
#            description('AAPL', 'ladkjsdlv')
        with self.assertRaises(ValueError):
            f('aoijdsoigjs')
            
            





import os as _os

global p
p = _os.path.abspath(_os.path.join(_os.path.dirname(__file__), _os.pardir))
global datapath
datapath = _os.path.join(p, 'database')


def create_folder(stock, path='default'):
    if path=='default':
        ppath = _os.path.join(datapath, stock)
    else:
        ppath = _os.path.join(p, path, stock)
    if _os.path.isdir(ppath):  # If dir existed
        pass
    else:
        _os.makedirs(ppath)  # Create directory


def exist(stock, file, update=False):
    p1 = _os.path.join(datapath, stock, stock)
    # if it needs update，then view it as not existing
    # and the save_file func handles the duplication
    if update:
        return False
    
    if isinstance(file, str):
        path = p1 + '_' + file + '.csv'
        return _os.path.exists(path)
    elif isinstance(file, list):
        paths = [p1 + '_' + f + '.csv' for f in file]
        r = '&'.join([str(_os.path.exists(pp)) for pp in paths])
        return eval(r)
    else:
        raise TypeError("Parameter 'file' should be either a string or a list of strings")



import random
import hashlib
import urllib
import json
import requests


def _t_util(text, t, f):
    # if not (t in Lans().values() and f in Lans().values()):
    #    raise ValueError("Inaapropriate language set, use function Lans() to see the abbreviations of languages")
    # Using Baidu translate API
    url = 'http://api.fanyi.baidu.com/api/trans/vip/translate'
    app_id = '20190321000279637'
    secretKey = 'mCoDy3N0ANeB7MVtUsDT'  # My own
    salt = random.randint(12345, 98765)

    temp = app_id + text + str(salt) + secretKey
    sign = hashlib.md5(temp.encode()).hexdigest()

    r_url = url + '?q=' + urllib.parse.quote(text) + '&from=' + f + '&to=' + t + '&appid=' + app_id + '&salt=' + str(
        salt) + '&sign=' + sign

    r = requests.get(r_url, timeout=7).content.decode('utf-8')
    data = json.loads(r)
    result = str(data['trans_result'][0]['dst'])

    return result


def _translate(text, t='en'):
    if t == 'en':
        return text
    trans = _t_util(text, t, f='en')
    return trans



0,1
Beta (3Y Monthly),0.89
52-Week Change,-4.39%
S&P500 52-Week Change,4.92%
52 Week High,233.47
52 Week Low,142.00
50-Day Moving Average,198.13
200-Day Moving Average,179.17
Avg Vol (3 month),29.6M
Avg Vol (10 day),31.84M
Shares Outstanding,4.6B
Float,4.35B
% Held by Insiders,0.07%
% Held by Institutions,60.95%
"Shares Short (Apr 30, 2019)",52.67M
"Short Ratio (Apr 30, 2019)",2.19
"Short % of Float (Apr 30, 2019)",1.11%
"Short % of Shares Outstanding (Apr 30, 2019)",1.14%
"Shares Short (prior month Mar 29, 2019)",67.79M



Name,Title,Pay,Exercised,Year Born
Mr. Jeffrey P. Bezos,"Founder, Chairman, Pres & CEO",1.68M,N/A,1964
Mr. Brian T. Olsavsky,Sr. VP & CFO,163.2k,N/A,1964
Mr. Jeffrey M. Blackburn,Sr. VP of Bus. Devel.,178.5k,N/A,1970
Mr. Jeffrey A. Wilke,Chief Exec. Officer of Worldwide Consumer,255.61k,N/A,1967
Mr. Andrew R. Jassy,Chief Exec. Officer of Amazon Web Services,266.23k,N/A,1968



Earnings Estimate,Current Qtr. (Jun 2019),Next Qtr. (Sep 2019),Current Year (2019)
No. of Analysts,41,40,46
Avg. Estimate,5.49,6.66,27.26
Low Estimate,4.34,4.38,21.53
High Estimate,6.77,8.28,32.68
Year Ago EPS,5.07,5.75,20.14



Name,Title,Pay,Exercised,Year Born
Mr. Timothy D. Cook,CEO & Director,15.68M,N/A,1961
Mr. Luca Maestri,CFO & Sr. VP,5.02M,N/A,1964
Mr. Jeffrey E. Williams,Chief Operating Officer,5.05M,N/A,1964
Ms. Katherine L. Adams,"Sr. VP, Gen. Counsel & Sec.",5.27M,N/A,1964
Mr. Chris Kondo,Sr. Director of Corp. Accounting,N/A,N/A,N/A
